Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports System.Windows.Resources
Imports System.Xml.Serialization
Imports Microsoft.Phone
Imports Microsoft.Phone.BackgroundTransfer
Imports Microsoft.Phone.Controls

Namespace BigPicture
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		Private Const transferDirectory As String = "/shared/transfers" ' Required for background transfer
		Private Const imagesDirectory As String = "/images"
		Private Const thumbnailsDirectory As String = PictureInfo.ThumbnailsDirectory

		Private pictureLibrary As PictureLibrary
		Private transferPreferences As TransferPreferences = TransferPreferences.None

		Public Sub New()
			InitializeComponent()

			' Load picture library information for ListBox
			Dim uri As New Uri("PictureLibrary.xml", UriKind.Relative)
			Dim resourceInfo As StreamResourceInfo = Application.GetResourceStream(uri)
			Dim reader As New StreamReader(resourceInfo.Stream)
			Dim xml As New XmlSerializer(GetType(PictureLibrary))
			pictureLibrary = TryCast(xml.Deserialize(reader), PictureLibrary)
			listBox.ItemsSource = pictureLibrary.Pictures

			Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
				' Make sure directories exist
				If Not isoStorage.DirectoryExists(transferDirectory) Then
					isoStorage.CreateDirectory(transferDirectory)
				End If

				If Not isoStorage.DirectoryExists(imagesDirectory) Then
					isoStorage.CreateDirectory(imagesDirectory)
				End If

				If Not isoStorage.DirectoryExists(thumbnailsDirectory) Then
					isoStorage.CreateDirectory(thumbnailsDirectory)
				End If

				' Get files already downloaded
				Dim filenames() As String = isoStorage.GetFileNames(imagesDirectory & "/*.*")

                For Each pictureInfo In pictureLibrary.Pictures
                    pictureInfo.IsDownloaded = filenames.Contains(pictureInfo.Filename)
                Next pictureInfo
			End Using
		End Sub

		Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
			' Might be returning from TransferSettingsPage
			Dim applicationSettings As IsolatedStorageSettings = IsolatedStorageSettings.ApplicationSettings

			If applicationSettings.Contains(TransferSettingsPage.Key) Then
				transferPreferences = CType(applicationSettings(TransferSettingsPage.Key), TransferPreferences)
			End If

			' Get current requests
            For Each request In BackgroundTransferService.Requests
                Dim index As Integer = Int32.Parse(request.Tag)
                Dim pictureInfo As PictureInfo = pictureLibrary.Pictures(index)
                pictureInfo.DownloadRequestId = request.RequestId

                If request.TransferStatus = TransferStatus.Completed Then
                    TransferCompleted(pictureInfo, request)
                Else
                    pictureInfo.IsDownloading = True
                    pictureInfo.DownloadProgress = 100.0 * request.BytesReceived / request.TotalBytesToReceive
                    pictureInfo.DownloadStatus = FormatTransferStatus(request.TransferStatus)
                    AddHandler request.TransferProgressChanged, AddressOf OnBackgroundTransferRequestProgressChanged
                    AddHandler request.TransferStatusChanged, AddressOf OnBackgroundTransferRequestStatusChanged
                End If
            Next request
			MyBase.OnNavigatedTo(args)
		End Sub

		Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
            For Each request In BackgroundTransferService.Requests
                Dim index As Integer = Int32.Parse(request.Tag)
                Dim pictureInfo As PictureInfo = pictureLibrary.Pictures(index)

                ' Remove event handlers
                RemoveHandler request.TransferProgressChanged, AddressOf OnBackgroundTransferRequestProgressChanged
                RemoveHandler request.TransferStatusChanged, AddressOf OnBackgroundTransferRequestStatusChanged
            Next request
			MyBase.OnNavigatedFrom(args)
		End Sub

		Private Sub OnDownloadButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
			Dim btn As Button = TryCast(sender, Button)
			Dim pictureInfo As PictureInfo = TryCast(btn.DataContext, PictureInfo)

			Dim requestUri As New Uri(pictureInfo.Url)
			Dim locationUri As New Uri(transferDirectory & "/" & pictureInfo.Filename, UriKind.Relative)
			Dim request As New BackgroundTransferRequest(requestUri, locationUri)
			request.Method = "GET"
			request.Tag = pictureLibrary.Pictures.IndexOf(pictureInfo).ToString
			request.TransferPreferences = transferPreferences
            AddHandler request.TransferProgressChanged, AddressOf OnBackgroundTransferRequestProgressChanged
            AddHandler request.TransferStatusChanged, AddressOf OnBackgroundTransferRequestStatusChanged

            Try
                BackgroundTransferService.Add(request)
                pictureInfo.IsDownloading = True
                pictureInfo.DownloadProgress = 0
                pictureInfo.DownloadRequestId = request.RequestId
            Catch exc As Exception
                MessageBox.Show("Problem with download request: " & exc.Message, "Big Picture", MessageBoxButton.OK)
            End Try
        End Sub

		Private Sub OnCancelDownloadButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
			Dim btn As Button = TryCast(sender, Button)
			Dim pictureInfo As PictureInfo = TryCast(btn.DataContext, PictureInfo)
			Dim requestId As String = pictureInfo.DownloadRequestId
			Dim request As BackgroundTransferRequest = BackgroundTransferService.Find(requestId)
			BackgroundTransferService.Remove(request)
		End Sub

		Private Sub OnBackgroundTransferRequestStatusChanged(ByVal sender As Object, ByVal args As BackgroundTransferEventArgs)
			Dim request As BackgroundTransferRequest = args.Request
			Dim index As Integer = Int32.Parse(request.Tag)
			Dim pictureInfo As PictureInfo = pictureLibrary.Pictures(index)
			pictureInfo.DownloadStatus = FormatTransferStatus(request.TransferStatus)

			If request.TransferStatus = TransferStatus.Completed Then
				TransferCompleted(pictureInfo, request)
			End If
		End Sub

		Private Sub TransferCompleted(ByVal pictureInfo As PictureInfo, ByVal request As BackgroundTransferRequest)
			Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
				' Successful completion
				If request.TransferError Is Nothing Then
					' Move downloaded file to images directory
					isoStorage.MoveFile(transferDirectory & "/" & pictureInfo.Filename, imagesDirectory & "/" & pictureInfo.Filename)

					' Create thumbnail 
					Dim bitmap As WriteableBitmap = Nothing

					Using stream As IsolatedStorageFileStream = isoStorage.OpenFile(imagesDirectory & "/" & pictureInfo.Filename, FileMode.Open, FileAccess.Read)
						bitmap = PictureDecoder.DecodeJpeg(stream, 144, 144)
					End Using

					' Save thumbnail
					Using stream As IsolatedStorageFileStream = isoStorage.CreateFile(thumbnailsDirectory & "/" & pictureInfo.Filename)
						bitmap.SaveJpeg(stream, bitmap.PixelWidth, bitmap.PixelHeight, 0, 80)
					End Using
				' Error (probably a Cancel so don't report it)
				Else
					If isoStorage.FileExists(transferDirectory & "/" & pictureInfo.Filename) Then
						isoStorage.DeleteFile(transferDirectory & "/" & pictureInfo.Filename)
					End If
				End If
			End Using

			pictureInfo.IsDownloading = False
			pictureInfo.IsDownloaded = request.TransferError Is Nothing

			If BackgroundTransferService.Find(pictureInfo.DownloadRequestId) IsNot Nothing Then
				BackgroundTransferService.Remove(request)
			End If
		End Sub

		Private Sub OnBackgroundTransferRequestProgressChanged(ByVal sender As Object, ByVal args As BackgroundTransferEventArgs)
			Dim request As BackgroundTransferRequest = args.Request
			Dim index As Integer = Int32.Parse(request.Tag)
			Dim pictureInfo As PictureInfo = pictureLibrary.Pictures(index)
			pictureInfo.DownloadProgress = 100.0 * request.BytesReceived / request.TotalBytesToReceive
		End Sub

		Private Sub ListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
			If listBox.SelectedItem IsNot Nothing Then
				Dim pictureInfo As PictureInfo = TryCast(listBox.SelectedItem, PictureInfo)

				If pictureInfo.IsDownloaded Then
					Dim destination As String = String.Format("/ViewPage.xaml?Title={0}&Filename={1}", pictureInfo.Title, imagesDirectory & "/" & pictureInfo.Filename)

					Me.NavigationService.Navigate(New Uri(destination, UriKind.Relative))
				End If
				listBox.SelectedItem = Nothing
			End If
		End Sub

		Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
			Dim btn As Button = TryCast(sender, Button)
			Dim pictureInfo As PictureInfo = TryCast(btn.DataContext, PictureInfo)

			Dim result As MessageBoxResult = MessageBox.Show("Delete " & pictureInfo.Title & "?", "Big Picture", MessageBoxButton.OKCancel)
            If result = MessageBoxResult.OK Then
                Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
                    isoStorage.DeleteFile(imagesDirectory & "/" & pictureInfo.Filename)
                    isoStorage.DeleteFile(thumbnailsDirectory & "/" & pictureInfo.Filename)
                    pictureInfo.IsDownloaded = False
                End Using
            End If
		End Sub

		Private Sub OnAppBarSettingsButtonClick(ByVal sender As Object, ByVal args As EventArgs)
			Me.NavigationService.Navigate(New Uri("/TransferSettingsPage.xaml", UriKind.Relative))
		End Sub

		Private Function FormatTransferStatus(ByVal transferStatus As TransferStatus) As String
			Select Case transferStatus
				Case TransferStatus.None
					Return ""

				Case TransferStatus.WaitingForExternalPower
					Return "Waiting for external power"

				Case TransferStatus.WaitingForExternalPowerDueToBatterySaverMode
					Return "Waiting for external power due to battery saver mode"

				Case TransferStatus.WaitingForNonVoiceBlockingNetwork
					Return "Waiting for non-voice-blocking network"

				Case TransferStatus.WaitingForWiFi
					Return "Waiting for WiFi"
			End Select
			Return transferStatus.ToString
		End Function
	End Class
End Namespace
