Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Windows
Imports System.Windows.Input
Imports System.Windows.Media.Animation
Imports System.Windows.Media.Imaging
Imports System.Windows.Navigation
Imports Microsoft.Devices.Sensors
Imports Microsoft.Phone
Imports Microsoft.Phone.Controls
Imports Microsoft.Xna.Framework

Namespace BigPicture
	Partial Public Class ViewPage
		Inherits PhoneApplicationPage

		Private motion As Motion
		Private screenWidth, screenHeight, maxDimension As Double
		Private imageWidth, imageHeight As Integer
        Private _title, filename As String
		Private zoomInScale As Double
		Private rotation As Double
		Private isZoomView As Boolean

		' Dependency property for zoom-in transition animation
		Public Shared ReadOnly InterpolationFactorProperty As DependencyProperty = DependencyProperty.Register("InterpolationFactor", GetType(Double), GetType(ViewPage), New PropertyMetadata(AddressOf OnInterpolationFactorChanged))

		Public Sub New()
			InitializeComponent()

			AddHandler Loaded, AddressOf OnLoaded

			If Motion.IsSupported Then
				motion = New Motion
				motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(33)
				AddHandler motion.CurrentValueChanged, AddressOf OnMotionCurrentValueChanged
			End If
		End Sub

		Public Property InterpolationFactor As Double
			Set(ByVal value As Double)
				SetValue(InterpolationFactorProperty, value)
			End Set
			Get
				Return CDbl(GetValue(InterpolationFactorProperty))
			End Get
		End Property

		Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
			Dim parameters As IDictionary(Of String, String) = Me.NavigationContext.QueryString

			If parameters.ContainsKey("Title") Then
                _title = parameters("Title").ToUpper
			End If

			If parameters.ContainsKey("Filename") Then
				filename = parameters("Filename")
			End If

			' Start the Motion sensor if it's available
			If motion IsNot Nothing Then
				Try
					motion.Start()
				Catch
				End Try
			End If
			MyBase.OnNavigatedTo(args)
		End Sub

		Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
			If motion IsNot Nothing Then
				motion.Stop()
			End If
			MyBase.OnNavigatedFrom(args)
		End Sub

		Private Sub OnLoaded(ByVal sender As Object, ByVal args As RoutedEventArgs)
			' Save the screen dimensions
			screenWidth = Me.ActualWidth
			screenHeight = Me.ActualHeight
			maxDimension = Math.Max(screenWidth, screenHeight)

			' Initialize some values
			outlineBorder.Width = screenWidth
			outlineBorder.Height = screenHeight
			borderTransform.CenterX = screenWidth / 2
			borderTransform.CenterY = screenHeight / 2

			' Load the image from isolated storage
			If image.Source Is Nothing AndAlso filename IsNot Nothing Then
				Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
					Dim bitmap As WriteableBitmap = Nothing

					Using stream As IsolatedStorageFileStream = isoStorage.OpenFile(filename, FileMode.Open, FileAccess.Read)
						Try
							bitmap = PictureDecoder.DecodeJpeg(stream)
						Catch exc As Exception
							MessageBox.Show("Error creating bitmap: " & exc.Message, "Big Picture", MessageBoxButton.OK)
						End Try
					End Using
					statusText.Visibility = Visibility.Collapsed
					image.Source = bitmap

					' Save image dimensions
					imageWidth = bitmap.PixelWidth
					imageHeight = bitmap.PixelHeight
                    titleText.Text = String.Format("{0} ({1}" & ChrW(&HD7).ToString & "{2})", _title, imageWidth, imageHeight)

					zoomInScale = Math.Min(screenWidth / imageWidth, screenHeight / imageHeight)
					UpdateImageTransforms()
				End Using
			End If
		End Sub

		Protected Overrides Sub OnTap(ByVal args As GestureEventArgs)
			' Animate the InterpolationFactor property
			Dim doubleAnimation As DoubleAnimation = New DoubleAnimation With {.To = If(isZoomView, 0, 1)}
			Storyboard.SetTarget(doubleAnimation, Me)
			Storyboard.SetTargetProperty(doubleAnimation, New PropertyPath(ViewPage.InterpolationFactorProperty))

            Dim _storyboard As New Storyboard
            _storyboard.Children.Add(doubleAnimation)
            _storyboard.Begin()
			isZoomView = isZoomView Xor True
			MyBase.OnTap(args)
		End Sub

		Private Sub OnMotionCurrentValueChanged(ByVal sender As Object, ByVal args As SensorReadingEventArgs(Of MotionReading))
				' If the image isn't loaded yet, don't do anything
				' Get the rotation matrix & convert to horizontal coordinates
				' Set the transform center on the Image element
				' Set the translation on the Border element
				' Get rotation from Tilt
            Me.Dispatcher.BeginInvoke(Sub()
                                          If image.Source Is Nothing Then
                                              Exit Sub
                                          End If
                                          Dim matrix As Matrix = args.SensorReading.Attitude.RotationMatrix
                                          Dim horzCoord As HorizontalCoordinate = HorizontalCoordinate.FromMotionMatrix(matrix)
                                          imageTransform.CenterX = (imageWidth + maxDimension) * (180 + horzCoord.Azimuth) / 360 - maxDimension / 2
                                          imageTransform.CenterY = (imageHeight + maxDimension) * (90 - horzCoord.Altitude) / 180 - maxDimension / 2
                                          borderTransform.TranslateX = imageTransform.CenterX - screenWidth / 2
                                          borderTransform.TranslateY = imageTransform.CenterY - screenHeight / 2
                                          rotation = -horzCoord.Tilt
                                          UpdateImageTransforms()
                                      End Sub)
		End Sub

		Private Shared Sub OnInterpolationFactorChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
			TryCast(obj, ViewPage).UpdateImageTransforms
		End Sub

		Private Sub UpdateImageTransforms()
			' If being zoomed out, set scaling
			Dim interpolatedScale As Double = 1 + InterpolationFactor * (zoomInScale - 1)
			imageTransform.ScaleY = interpolatedScale
			imageTransform.ScaleX = imageTransform.ScaleY

			' Move transform center to screen center
			imageTransform.TranslateX = screenWidth / 2 - imageTransform.CenterX
			imageTransform.TranslateY = screenHeight / 2 - imageTransform.CenterY

			' If being zoomed out, adjust for scaling
			imageTransform.TranslateX -= InterpolationFactor * (screenWidth / 2 - zoomInScale * imageTransform.CenterX)
			imageTransform.TranslateY -= InterpolationFactor * (screenHeight / 2 - zoomInScale * imageTransform.CenterY)

			' If being zoomed out, center image in screen
			imageTransform.TranslateX += InterpolationFactor * (screenWidth - zoomInScale * imageWidth) / 2
			imageTransform.TranslateY += InterpolationFactor * (screenHeight - zoomInScale * imageHeight) / 2

			' Set border thickness
			outlineBorder.BorderThickness = New Thickness(2 / interpolatedScale)
			outlineRectangle.StrokeThickness = 2 / interpolatedScale

			' Set rotation on image and border
			imageTransform.Rotation = (1 - InterpolationFactor) * rotation
			borderTransform.Rotation = -rotation
		End Sub
	End Class
End Namespace
